

import { Component, OnInit } from '@angular/core';
import { IonicModule, ToastController } from '@ionic/angular';
import { FormsModule } from '@angular/forms';
import { CommonModule } from '@angular/common';
import { TagInputComponent } from '../tag-input/tag-input.component';
import { addIcons } from 'ionicons';
import { barbellOutline, personOutline, square, alarmOutline, arrowBack } from 'ionicons/icons';
import { DalleOptions, FmodeChatCompletion, MarkdownPreviewModule } from 'fmode-ng';
import { LoadingController } from '@ionic/angular';
import { AlertController, IonThumbnail, IonBadge, IonButton, IonButtons, IonCard, IonCardContent, IonCardHeader, IonCardSubtitle, IonCardTitle, IonCol, IonContent, IonGrid, IonHeader, IonIcon, IonImg, IonInput, IonItem, IonLabel, IonList, IonRow, IonSegment, IonSegmentButton, IonSelect, IonSelectOption, IonToolbar, IonTitle, IonSpinner, IonTextarea } from '@ionic/angular/standalone';
import { NavController } from '@ionic/angular';
import { Router } from '@angular/router';
import { CloudObject, CloudQuery, CloudUser } from 'src/lib/ncloud';
import { IonBackButton } from '@ionic/angular/standalone';
@Component({
  selector: 'app-kuai',
  templateUrl: './kuai.component.html',
  styleUrls: ['./kuai.component.scss'],
  standalone: true,
  imports: [
    IonTextarea,
    IonSpinner,
    IonTitle,
    IonBackButton,
    MarkdownPreviewModule,
    IonSelectOption,
    IonSelect,
    IonThumbnail,
    IonCardSubtitle,
    IonImg,
    IonCard,
    IonCardTitle,
    IonCardHeader,
    IonCardContent,
    IonButtons,
    IonItem,
    IonList,
    IonIcon,
    FormsModule,
    CommonModule,
    IonHeader,
    IonToolbar,
    IonContent,
    IonSegment,
    IonSegmentButton,
    IonGrid,
    IonRow,
    IonCol,
    IonButton,
    IonLabel,
    IonBadge,
    IonInput,
    TagInputComponent
  ]  // 添加 CommonModule
})
export class KuaiComponent implements OnInit {
  selectedTags: string[] = [];
  exercisePreference: string = '';
  workoutFrequency: string = '';
  height: number | null = null;
  weight: number | null = null;
  age: number | null = null;
  goalDescription: string = '';
  generatedPlan: string = '';  // 用来存储生成的健身计划
  isGenerating: boolean = false;  // 是否正在生成计划
  messageList: any[] = [];  // 存储消息列表
  constructor(private toastController: ToastController,
    private loadingController: LoadingController,
    private alertController: AlertController,
    private navController: NavController,
    private router: Router,
    private navCtrl: NavController
  ) {
    addIcons({ personOutline, barbellOutline, alarmOutline, square, arrowBack });
  }
  ngOnInit() { }

  onTagsChanged(tags: string[]) {
    this.selectedTags = tags;
    console.log('当前标签:', this.selectedTags);
  }

  onGoalDescriptionChange(event: any) {
    this.goalDescription = event.detail.value;
  }

  onExercisePreferenceChange(event: any) {
    this.exercisePreference = event.detail.value;
  }

  onWorkoutFrequencyChange(event: any) {
    this.workoutFrequency = event.detail.value;
  }

  onHeightChange(event: any) {
    this.height = event.detail.value;
  }

  onWeightChange(event: any) {
    this.weight = event.detail.value;
  }

  onAgeChange(event: any) {
    this.age = event.detail.value;
  }

  // 显示toast提示
  async showToast(message: string) {
    const toast = await this.toastController.create({
      message: message,
      duration: 2000,
      position: 'top',
      color: 'danger',
    });
    toast.present();
  }

  async generatePlan() {
    // 检查身高和体重是否为空
    if (!this.height || !this.weight) {
      // 如果为空，显示提示信息
      await this.showToast('身高或体重为空，无法生成个人健身计划');
      return; // 退出函数，不继续生成健身计划
    }
    const loading = await this.loadingController.create({
      message: '生成计划中...',
      duration: 30000 // 加载时长可以根据实际需求调整
    });
    await loading.present();
    this.generatedPlan = '';
    this.messageList = [];
    this.isGenerating = true;

    console.log('生成健身计划', {
      tags: this.selectedTags,
      exercisePreference: this.exercisePreference,
      workoutFrequency: this.workoutFrequency,
      height: this.height,
      weight: this.weight,
      age: this.age,
      goalDescription: this.goalDescription
    });

    let prompt = `您作为一名专业的健身计划定制大师，请帮我根据以下情况制定健身计划(健身计划请给每天的运动标上序号)。关键词：${this.selectedTags.join(",")},目标描述:${this.goalDescription},运动偏好:${this.exercisePreference},
    健身频率每周:${this.workoutFrequency},身高:${this.height}cm,体重:${this.weight}kg,年龄:${this.age},注意：只需给出训练日计划，训练日只做四个运动`;

    let messageList = new FmodeChatCompletion([
      { role: "system", content: '' },
      { role: "user", content: prompt }
    ]);

    messageList.sendCompletion().subscribe((message: any) => {
      console.log(message.content);
      this.generatedPlan = message.content;

      if (message?.complete) {
        this.isGenerating = false;
        this.getJson()
        loading.dismiss();
      }
    });
  }
  public gptre = ""
  title: string = "123"
  chatID = ""
  complete: boolean = false
  planArray: any
  JSONcomplete: boolean = false
  JSONdes = ""
  JSONobject: { [key: string]: string } = {}
  getJson() {
    let promt = `请你以以下json格式分每天来整合文段内容：
    {
    "date":"周几运动",
    "srcId": "plan001等等",
    "trainingPart":"训练部位",
    "trainingItems":"四个运动项目,以下面形式给出,形式如下"[
  {
    "item": "引体向上",
    "reps": 8,
    "sets": 4
  },
  {
    "item": "划船",
    "reps": 10,
    "sets": 3
  },
  {
    "item": "背部伸展",
    "reps": 12,
    "sets": 3
  },
  {
    "item": "拉力器划船",
    "reps": 12,
    "sets": 3
  }
]"
}
    内容如下：${this.generatedPlan}
`

    let completion = new FmodeChatCompletion([
      { role: "system", content: "" },
      { role: "user", content: promt }
    ])
    completion.sendCompletion().subscribe((message: any) => {
      console.log(message.content)
      this.JSONdes = message.content;

      
    });
  }
  async showConfirmationAlert() {
    const alert = await this.alertController.create({
      header: '确认生成计划',
      message: ` <div style="white-space: pre-wrap;">
        <!-- 使用 fm-markdown-preview 渲染 Markdown 格式的内容 -->
        <fm-markdown-preview class="content-style" [content]="generatedPlan"></fm-markdown-preview>
      </div>`,
      buttons: [
        {
          text: '放弃',
          role: 'cancel',
          handler: () => {
            console.log('用户选择放弃');
          }
        },
        {
          text: '确认',
          handler: () => {
            // 这里可以调用 API 将数据存到数据库
            console.log('用户确认', this.generatedPlan);
            this.savePlanToDatabase(this.generatedPlan);
          }
        }
      ]
    });
    await alert.present();
  }
  async savePlanToDatabase(plan: string) {

    let currentUser = new CloudUser().toPointer();  // 获取当前用户实例
    console.log(currentUser)
    console.log('计划保存到数据库', plan);

    // 查询当前用户的所有计划
    const cloudQuery = new CloudQuery("fitPlan");
    // 过滤出与当前用户关联的所有计划
    cloudQuery.equalTo("user", currentUser)
    try {
      const userPlans = await cloudQuery.find();  // 获取当前用户的所有计划
      console.log("找到的计划数量:", userPlans.length);

      // 删除当前用户的所有计划
      for (const planObj of userPlans) {
        await planObj.destroy();  // 删除每个计划
        console.log("已删除计划:", planObj);
      }
      this.planArray.forEach((plan: any) => {
        plan.detail = this.generatedPlan; // 添加额外的计划内容
        console.log(plan);

        // 准备插入数据库的数据对象
        let newPlan = {
          "date": plan['date'],
          "srcId": plan['srcId'],
          "trainingPart": plan['trainingPart'],
          "trainingItems": plan['trainingItems'],
          "user": currentUser,  // 将当前用户指针关联到计划
        };
        // 创建新的 CloudObject 实例
        let newObject = new CloudObject("fitPlan");
        newObject.set(newPlan);
        newObject.save()
          .then(() => {
            console.log("计划保存成功");
          })
          .catch((error: any) => {
            console.error("保存计划时发生错误", error);
          });
      });
      this.JSONcomplete = true;
    } catch (error) {
      console.error("查询或删除计划时发生错误", error);
    }
    this.goToPage("tab2")
  }

  goBack() {
    this.navCtrl.back();
  }
  goToPage(page: string) {
    // 更新选中的tab
    this.router.navigate([`/tabs/${page}`]);  // 然后再进行路由跳转
  }
}
